<?php

namespace Drupal\mautic_paragraph_commerce\Entity;

use Drupal\commerce\ConditionGroup;
use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Config\Entity\ConfigEntityBase;
use Drupal\mautic_paragraph_commerce\MauticCommerceConditionInterface;

/**
 * Defines the mautic commerce condition entity type.
 *
 * @ConfigEntityType(
 *   id = "mautic_commerce_condition",
 *   label = @Translation("Mautic Commerce Condition"),
 *   label_collection = @Translation("Mautic Commerce Conditions"),
 *   label_singular = @Translation("mautic commerce condition"),
 *   label_plural = @Translation("mautic commerce conditions"),
 *   label_count = @PluralTranslation(
 *     singular = "@count mautic commerce condition",
 *     plural = "@count mautic commerce conditions",
 *   ),
 *   handlers = {
 *     "list_builder" = "Drupal\mautic_paragraph_commerce\MauticCommerceConditionListBuilder",
 *     "form" = {
 *       "add" = "Drupal\mautic_paragraph_commerce\Form\MauticCommerceConditionForm",
 *       "edit" = "Drupal\mautic_paragraph_commerce\Form\MauticCommerceConditionForm",
 *       "delete" = "Drupal\Core\Entity\EntityDeleteForm"
 *     }
 *   },
 *   config_prefix = "mautic_commerce_condition",
 *   admin_permission = "administer mautic_commerce_condition",
 *   links = {
 *     "collection" = "/admin/commerce/mautic-commerce-conditions",
 *     "add-form" = "/admin/commerce/mautic-commerce-conditions/add",
 *     "edit-form" = "/admin/commerce/mautic-commerce-conditions/{mautic_commerce_condition}",
 *     "delete-form" = "/admin/commerce/mautic-commerce-conditions/{mautic_commerce_condition}/delete"
 *   },
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "label",
 *     "uuid" = "uuid"
 *   },
 *   config_export = {
 *     "id",
 *     "label",
 *     "description",
 *     "settings"
 *   }
 * )
 */
class MauticCommerceCondition extends ConfigEntityBase implements MauticCommerceConditionInterface {

  /**
   * The mautic commerce condition ID.
   *
   * @var string
   */
  protected $id;

  /**
   * The mautic commerce condition label.
   *
   * @var string
   */
  protected $label;

  /**
   * The mautic commerce condition status.
   *
   * @var bool
   */
  protected $status;

  /**
   * The mautic_commerce_condition description.
   *
   * @var string
   */
  protected $description;

  /**
   * The mautic commerce condition settings.
   *
   * @var array
   */
  protected $settings = [];

  /**
   * {@inheritdoc}
   */
  public function setLabel($value) {
    if (!empty($value)) {
      $this->label = $value;
    }
    else {
      throw new \Exception('You must choose a Mautic segment.');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getSettings($setting_id = '') {
    if ($setting_id == '') {
      return $this->settings;
    }

    if (empty($this->settings[$setting_id])) {
      if (array_key_exists($setting_id, $this->settings)) {
        if ($this->settings[$setting_id] === 0 || $this->settings[$setting_id] === "0") {
          $value = 0;
        }
        else {
          $value = NULL;
        }
      }
      else {
        $value = NULL;
      }
    }
    else {
      $value = $this->settings[$setting_id];
    }

    return $value;
  }

  /**
   * {@inheritdoc}
   */
  public function setSetting($setting_id, $value) {
    if (!empty($setting_id)) {
      $this->settings[$setting_id] = $value;
    }
    else {
      throw new \Exception('You must enter a value to the settingId');
    }
  }

  /**
   * {@inheritdoc}
   */
  public function getConditions() {

    $plugin_manager = \Drupal::service('plugin.manager.commerce_condition');
    $conditions = [];
    $commerce_conditions = $this->getSettings('commerce_conditions');
    if (!is_null($commerce_conditions)) {
      foreach ($commerce_conditions as $commerce_condition) {
        $commerce_condition = $plugin_manager->createInstance($commerce_condition['plugin'], $commerce_condition['configuration']);
        $conditions[] = $commerce_condition;
      }
    }
    return $conditions;

  }

  /**
   * {@inheritdoc}
   */
  public function process(OrderInterface $order) {
    $result = FALSE;

    $conditions = $this->getConditions();

    if (\Drupal::moduleHandler()->moduleExists('commerce_conditions_plus')) {
      $sut = \Drupal::getContainer()->get('commerce_conditions_plus.conditions_evaluator');
      $result = $sut->execute($conditions, $this->getSettings('operator'), [
        'commerce_order' => $order,
      ]);
    }
    else {
      $order_conditions = array_filter($conditions, function ($commerce_condition) {
        /** @var \Drupal\commerce\Plugin\Commerce\Condition\ConditionInterface $commerce_condition */
        return $commerce_condition->getEntityTypeId() === 'commerce_order';
      });
      $order_conditions = new ConditionGroup($order_conditions, $this->getSettings('operator'));
      $result = $order_conditions->evaluate($order);
    }

    return $result;
  }

}
