<?php

namespace Drupal\mautic_paragraph_commerce;

use Drupal\commerce_order\Entity\OrderInterface;
use Drupal\Core\Config\Entity\ConfigEntityInterface;

/**
 * Provides an interface defining a mautic commerce condition entity type.
 */
interface MauticCommerceConditionInterface extends ConfigEntityInterface {

  /**
   * Return the Mautic commerce condition settings.
   *
   * @param string $setting_id
   *   the settings ID for the config entity.
   *
   * @return array|string
   *   The item settings.
   */
  public function getSettings($setting_id = '');

  /**
   * Set a value to the Mautic commerce condition settings.
   *
   * @param string $setting_id
   *   The setting id.
   * @param mixed $value
   *   The value to be set on Item settings.
   *
   * @throws \Exception
   */
  public function setSetting($setting_id, $value);

  /**
   * Gets the Mautic commerce conditions.
   *
   * @return \Drupal\commerce\Plugin\Commerce\Condition\ConditionInterface[]
   *   The payment gateway conditions.
   */
  public function getConditions();

  /**
   * Process the Mautic commerce condition.
   *
   * @param \Drupal\commerce_order\Entity\OrderInterface $order
   *   The commerce order.
   *
   * @return bool
   *   Boolean value that indicates if the condition is true.
   */
  public function process(OrderInterface $order);

}
