<?php

namespace Drupal\mautic_paragraph_commerce;

use Drupal\Core\Config\Entity\ConfigEntityListBuilder;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Entity\EntityRepositoryInterface;
use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a listing of mautic commerce conditions.
 */
class MauticCommerceConditionListBuilder extends ConfigEntityListBuilder {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The entity repository.
   *
   * @var \Drupal\Core\Entity\EntityRepositoryInterface
   */
  protected $entityRepository;

  /**
   * Constructs a CropTypeListBuilder object.
   *
   * @param \Drupal\Core\Entity\EntityTypeInterface $entity_type
   *   The entity type definition.
   * @param \Drupal\Core\Entity\EntityStorageInterface $storage
   *   The entity storage class.
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Entity\EntityRepositoryInterface $entity_repository
   *   The entity repository.
   */
  public function __construct(EntityTypeInterface $entity_type, EntityStorageInterface $storage, EntityTypeManagerInterface $entity_type_manager, EntityRepositoryInterface $entity_repository) {
    parent::__construct($entity_type, $storage);
    $this->entityTypeManager = $entity_type_manager;
    $this->entityRepository = $entity_repository;
  }

  /**
   * {@inheritdoc}
   */
  public static function createInstance(ContainerInterface $container, EntityTypeInterface $entity_type) {
    return new static(
      $entity_type,
      $container->get('entity_type.manager')->getStorage($entity_type->id()),
      $container->get('entity_type.manager'),
      $container->get('entity.repository')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildHeader() {
    $header['sid'] = $this->t('SID');
    $header['mautic_segment'] = $this->t('Mautic Segment');
    $header['commerce_promotions'] = $this->t('Promotions');
    $header['commerce_conditions'] = $this->t('Conditions');
    $header['marketing_terms'] = $this->t('Marketing Terms');
    $header['status'] = $this->t('Status');
    return $header + parent::buildHeader();
  }

  /**
   * {@inheritdoc}
   */
  public function buildRow(EntityInterface $entity) {
    /** @var \Drupal\mautic_paragraph_commerce\MauticCommerceConditionInterface $entity */
    $rules_settings = [];
    $rules_settings_conf = [];
    $commerce_conditions = $entity->getSettings('commerce_conditions');
    $row['sid'] = $entity->getSettings('mautic_segment');
    $row['mautic_segment'] = $entity->label();

    if (isset($commerce_conditions)) {
      foreach ($commerce_conditions as $rules) {
        foreach ($rules['configuration'] as $key_configuration => $configuration) {
          if (is_array($configuration)) {
            foreach ($configuration as $conf) {
              if (is_array($conf)) {
                if (array_key_exists('product', $conf)) {
                  /** @var \Drupal\commerce_product\Entity\ProductInterface $commerce_product */
                  $commerce_product = $this->entityRepository
                    ->loadEntityByUuid('commerce_product', $conf['product']);
                  $rules_settings_conf[$rules['plugin']][] = ucfirst($commerce_product->label());
                }
                elseif (isset($conf[0]) && array_key_exists('country_code', $conf[0])) {
                  foreach ($conf as $teritory) {
                    $rules_settings_conf[$rules['plugin']][] = ucfirst($teritory['country_code']);
                  }
                }
              }
              else {
                if ($key_configuration === 'stores') {
                  /** @var \Drupal\commerce_store\Entity\StoreInterface $store */
                  $store = $this->entityRepository
                    ->loadEntityByUuid('commerce_store', $conf);
                  $rules_settings_conf[$rules['plugin']][] = ucfirst($store->label());
                }
                elseif ($key_configuration === 'entities') {
                  /** @var \Drupal\commerce_product\Entity\ProductVariationInterface $commerce_product_variation */
                  $commerce_product_variation = $this->entityRepository
                    ->loadEntityByUuid('commerce_product_variation', $conf);
                  $rules_settings_conf[$rules['plugin']][] = ucfirst($commerce_product_variation->label());
                }
                elseif ($key_configuration === 'terms') {
                  /** @var \Drupal\taxonomy\Entity\Term $terms */
                  $terms = $this->entityRepository
                    ->loadEntityByUuid('taxonomy_term', $conf);
                  $rules_settings_conf[$rules['plugin']][] = ucfirst($terms->label());
                }
                else {
                  $rules_settings_conf[$rules['plugin']][] = ucfirst($conf);
                }
              }

            }
          }
          foreach ($rules_settings_conf as $key => $value) {
            if ($key == $rules['plugin']) {
              $rules_settings[$rules['plugin']] = ucfirst(preg_replace("/[-_]+/", " ", $rules['plugin'])) . ': ' . implode(' , ', $rules_settings_conf[$key]);
            }
          }
        }
      }
    }

    $filter = function ($tag) {
      return '<p>' . $tag . '</p>';
    };

    $promotions = [];
    $promotion_list = [];
    if ($entity->getSettings('commerce_promotions')) {
      $promotions = $this->entityTypeManager->getStorage('commerce_promotion')->loadMultiple($entity->getSettings('commerce_promotions'));
    }
    foreach ($promotions as $promotion) {
      $promotion_list[] = $promotion->label();
    }
    $promotions_markup = array_map($filter, $promotion_list);
    $row_promotion = implode(' ', $promotions_markup);
    $row['commerce_promotions'] = [
      'data' => [
        '#markup' => $row_promotion,
      ],
    ];

    $commerce_conditions_markup = array_map($filter, $rules_settings);
    $rules = implode(' ', $commerce_conditions_markup);
    $row['commerce_conditions'] = [
      'data' => [
        '#markup' => $rules,
      ],
    ];

    $row['marketing_terms'] = $entity->getSettings('exclude_from_marketing_terms') ? $this->t('Excluded') : '';

    $row['status'] = $entity->status() ? $this->t('Enabled') : $this->t('Disabled');

    return $row + parent::buildRow($entity);
  }

}
